#include "testlib.h"
#include <optional>
#include <cassert>

using namespace std;

const char* MODE_CORRECT = "c";
const char* MODE_CORRECT_2 = "c2"; // correct, but you can move to get another solution
const char* MODE_IMPOSSIBLE = "i";
const char* MODE_SOLVABLE = "s";
const char* MODE_SOLVABLE_2 = "s2"; // solvable with more than 1 solution
const char* MODE_IMPOSSIBLE_LZ = "ilz"; // impossible, but solves with leading zero
const char* MODE_IMPOSSIBLE_ND = "ind"; // impossible, but solves by moving non-digit
const char* MODE_IMPOSSIBLE_11 = "i11"; // impossible, but solves with 11-digit numbers

enum Mode { CORRECT, CORRECT_2, IMPOSSIBLE, SOLVABLE, SOLVABLE_2, IMPOSSIBLE_LZ, IMPOSSIBLE_ND, IMPOSSIBLE_11 };

typedef long long number;
const number MAX_NUMBER = 9999999999LL;

struct Parser {
    string& s;
    int i;
    int len;

    Parser(string &_s) : s(_s), i(0), len(s.length()) {}

    optional<number> nextNumber() {
        number result = 0;
        int j = i;
        while (j < len && s[j] >= '0' && s[j] <= '9') {
            result = result * 10 + (s[j] - '0');
            if (result > MAX_NUMBER) return nullopt;
            j++;
        }
        if (j == i) return nullopt;
        if (result == 0 && j != i + 1) return nullopt;
        if (result != 0 && s[i] == '0') return nullopt;
        i = j;
        return result;
    }

    optional<number> nextExpression() {
        optional<number> next = nextNumber();
        if (!next) return nullopt;
        number result = next.value();
        while (i < len && (s[i] == '+' || s[i] == '-')) {
            char op = s[i];
            i++;
            next = nextNumber();
            if (!next) return nullopt;
            switch (op) {
                case '+': result += next.value(); break;
                case '-': result -= next.value(); break;
            }
        }
        return result;
    }

    optional<pair<number, number>> nextEquality() {
        optional<number> left = nextExpression();
        if (!left) return nullopt;
        if (i >= len || s[i] != '=') return nullopt;
        i++;
        optional<number> right = nextExpression();
        if (!right) return nullopt;
        if (i != len) return nullopt;
        return make_pair(left.value(), right.value());
    }
};

bool isWellformedEquality(string &s) {
    return Parser(s).nextEquality().has_value();
}

bool isCorrectEquality(string& s) {
    optional<pair<number, number>> eq = Parser(s).nextEquality();
    if (!eq) return false;
    return eq->first == eq->second;
}

number pow10(int n) {
    number result = 1;
    for (int i = 0; i < n; i++) result *= 10;
    return result;
}

int numSize(number x) {
    if (x < 0) x = -x;
    int n = 1;
    number pow10 = 10;
    while (x >= pow10) {
        pow10 *= 10;
        n++;
    }
    return n;
}

vector<number> genExprParts(int len, int minNL, int maxNL) {
    vector<number> result;
    while (len > 0) {
        bool first = result.empty();
        if (!first) len--;
        if (len < minNL) {
            result.clear();
            break;
        }
        int n = rnd.next(minNL, min(len, maxNL));
        number lo = n == 1 ? 0 : pow10(n - 1);
        number x = rnd.next(lo, pow10(n) - 1);
        if (!first && rnd.next(2) == 0) x = -x;
        result.push_back(x);
        len -= n;
    }
    return result;
}

number sum(const vector<number>& a) {
    number result = 0;
    for (int i = 0; i < a.size(); i++) result += a[i];
    return result;
}

string expr2str(const vector<number>& a) {
    string result;
    for (int i = 0; i < a.size(); i++) {
        if (i > 0 && a[i] >= 0) result += "+";
        result += to_string(a[i]);
    }
    return result;
}

string moveChar(string&s, int i, int j) {
    string t = s.substr(0, i) + s.substr(i + 1);
    return t.substr(0, j) + s[i] + t.substr(j);
}

bool isDigit(char c) {
    return c >= '0' && c <= '9';
}

int main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    int len = opt<int>(1);
    int minNL = opt<int>(2);
    int maxNL = opt<int>(3);
    string modeStr = opt(4);
    Mode mode;
    if (modeStr == MODE_CORRECT) {
        mode = CORRECT;
    } else if (modeStr == MODE_CORRECT_2) {
        mode = CORRECT_2;
    } else if (modeStr == MODE_IMPOSSIBLE) {
        mode = IMPOSSIBLE;
    } else if (modeStr == MODE_SOLVABLE) {
        mode = SOLVABLE;
    } else if (modeStr == MODE_SOLVABLE_2) {
        mode = SOLVABLE_2;
    } else if (modeStr == MODE_IMPOSSIBLE_LZ) {
        mode = IMPOSSIBLE_LZ;
    } else if (modeStr == MODE_IMPOSSIBLE_ND) {
        mode = IMPOSSIBLE_ND;
    } else if (modeStr == MODE_IMPOSSIBLE_11) {
        mode = IMPOSSIBLE_11;
        assert(maxNL == 11);
    } else {
        println("Invalid mode");
        return 1;
    }
    // generate correct equality
    while (true) {
        int genLen = mode == IMPOSSIBLE_LZ ? len - 1 : len;
        int eqpos = rnd.next(minNL, genLen - minNL - 1);
        vector<number> left = genExprParts(eqpos, minNL, maxNL);
        if (left.empty()) continue;
        vector<number> right = genExprParts(genLen - eqpos - 1, minNL, maxNL);
        if (right.empty()) continue;
        if (mode == IMPOSSIBLE_11) {
            // exactly one must have length=11
            int cnt11 = 0;
            for (number x : left) if (numSize(x) == 11) cnt11++;
            for (number x : right) if (numSize(x) == 11) cnt11++;
            if (cnt11 != 1) continue;
        }
        number leftSum = sum(left);
        number rightSum = sum(right);
        if (mode != IMPOSSIBLE) {
            number delta = rightSum - leftSum;
            bool fixLeft = rnd.next(2) == 0;
            vector<number>& fix = fixLeft ? left : right;
            if (!fixLeft) delta = -delta;
            int i = rnd.next((int)fix.size());
            number original = fix[i];
            number fixed = original + delta;
            if (i == 0 && fixed < 0) continue;
            if (numSize(original) != numSize(fixed)) continue;
            fix[i] = fixed;
        }
        string s = expr2str(left) + "=" + expr2str(right);
        if (mode == IMPOSSIBLE_11) {
            if (isWellformedEquality(s)) continue;
        } else {
            assert(isWellformedEquality(s));
        }
        int extraLeadingZero = -1;
        if (mode == IMPOSSIBLE_LZ) {
            // insert leading zero
            int i;
            do {
                i = rnd.next(genLen - 1);
            } while (!isDigit(s[i]) || (i > 0 && isDigit(s[i - 1])));
            s = s.substr(0, i) + '0' + s.substr(i);
            extraLeadingZero = i;
        }
        assert((int)s.size() == len);
        string originalSolution = s;
        if (mode == SOLVABLE || mode == SOLVABLE_2 || mode == IMPOSSIBLE_LZ || mode == IMPOSSIBLE_ND || mode == IMPOSSIBLE_11) {
            int i;
            if (mode == IMPOSSIBLE_LZ) {
                i = extraLeadingZero;
            } else if (mode == IMPOSSIBLE_ND) {
                do {
                    i = rnd.next(len);
                } while (isDigit(s[i]));
            } else if (mode == IMPOSSIBLE_11) {
                string t; // without i'th digit (some should be well-formed)
                do {
                    i = rnd.next(len);
                    t = s.substr(0, i) + s.substr(i + 1);
                } while (!isDigit(s[i]) || !isWellformedEquality(t));
            } else {
                do {
                    i = rnd.next(len);
                } while (!isDigit(s[i]));
            }
            string t;
            do {
                int j = rnd.next(len);
                t = moveChar(s, i, j);
            } while(!isWellformedEquality(t));
            if (isCorrectEquality(t)) continue;
            s = t;
        }
        if (mode == CORRECT_2 || mode == SOLVABLE_2) {
            bool found = false;
            for (int i = 0; i < len && !found; i++) {
                if (s[i] >= '0' && s[i] <= '9') {
                    for (int j = 0; j < len && !found; j++) {
                        string t = moveChar(s, i, j);
                        if (t != originalSolution && isCorrectEquality(t)) {
                            found = true;
                        }
                    }
                }
            }
            if (!found) continue;
        } else if (mode == IMPOSSIBLE || mode == IMPOSSIBLE_LZ || mode == IMPOSSIBLE_ND || mode == IMPOSSIBLE_11) {
            bool found = false;
            for (int i = 0; i < len && !found; i++) {
                if (s[i] >= '0' && s[i] <= '9') {
                    for (int j = 0; j < len && !found; j++) {
                        string t = moveChar(s, i, j);
                        if (isCorrectEquality(t)) {
                            found = true;
                        }
                    }
                }
            }
            if (found) continue;
        }
        println(s);
        return 0;
    } // while true
}
