private class Edge(
    val from: Int,
    val to: Int,
    var flow: Int,
    val cap: Int,
) {
    lateinit var revEdge: Edge
}

private class MaxFlow(n: Int) {
    val edges = Array(n) { mutableListOf<Edge>() }
    val visited = BooleanArray(n)

    fun addEdge(from: Int, to: Int, cap: Int): Edge {
        val e1 = Edge(from, to, 0, cap)
        val e2 = Edge(to, from, 0, 0)
        e1.revEdge = e2
        e2.revEdge = e1
        edges[from].add(e1)
        edges[to].add(e2)
        return e1
    }

    fun findMaxFlow(source: Int, sink: Int): Int {
        var ans = 0
        while (true) {
            visited.fill(false)
            val add = dfs(source, sink, Int.MAX_VALUE)
            if (add == 0) break
            ans += add
        }
        return ans
    }

    private fun dfs(v: Int, sink: Int, maxCap: Int): Int {
        if (v == sink) return maxCap
        visited[v] = true
        for (e in edges[v]) {
            if (e.flow == e.cap || visited[e.to]) continue
            val add = dfs(e.to, sink, minOf(e.cap - e.flow, maxCap))
            if (add == 0) continue
            e.flow += add
            e.revEdge.flow -= add
            return add
        }
        return 0
    }

    private fun dfsFind(v: Int, t: Int): Boolean {
        if (v == t) return true
        visited[v] = true
        for (e in edges[v]) {
            if (e.flow == e.cap || visited[e.to]) continue
            if (dfsFind(e.to, t)) return true
        }
        return false
    }

    fun hasPath(from: Int, to: Int): Boolean {
        visited.fill(false)
        return dfsFind(from, to)
    }
}

private fun readInt() = readln().toInt()
private fun readInts() = readln().split(' ').map { it.toInt() }

private class Block(val from: Int, val to: Int, val size: Int)

private fun inv(a: IntArray): IntArray {
    val res = IntArray(a.size)
    for (i in a.indices) {
        res[a[i]] = i
    }
    return res
}

private enum class Solution(val s: String) {
    NO_WAY("N"),
    MAYBE("?"),
    HAS_TO("Y"),
    ;
    
    override fun toString() = s
}

private fun solve(p1: IntArray, p2: IntArray, d1: IntArray, d2: IntArray): Array<Solution>? {
    val n = p1.size
    val inv1 = inv(p1)
    val inv2 = inv(p2)
    val blocks1 = buildBlocks(inv1, d1) ?: return null
    val blocks2 = buildBlocks(inv2, d2) ?: return null
    assert(blocks1.sumOf { it.size } == d1.size)
    assert(blocks2.sumOf { it.size } == d1.size)
    val where1 = buildBlockWhere(p1, blocks1)
    val where2 = buildBlockWhere(p2, blocks2)
    val g = MaxFlow(blocks1.size + blocks2.size + 2)
    val source = blocks1.size + blocks2.size
    val sink = source + 1
    val edges = (0 until n).map { element ->
        g.addEdge(where1[element], where2[element] + blocks1.size, 1)
    }
    for (i in blocks1.indices) {
        g.addEdge(source, i, blocks1[i].size)
    }
    for (i in blocks2.indices) {
        g.addEdge(blocks1.size + i, sink, blocks2[i].size)
    }
    val matched = g.findMaxFlow(source, sink)
    if (matched != d1.size) {
        return null
    }
    val ans = Array(n) { Solution.NO_WAY }
    for ((i, edge) in edges.withIndex()) {
        ans[i] = if (edge.flow > 0) {
            if (g.hasPath(edge.from, edge.to)) {
                Solution.MAYBE
            } else {
                Solution.HAS_TO
            }
        } else {
            if (g.hasPath(edge.to, edge.from)) {
                Solution.MAYBE
            } else {
                Solution.NO_WAY
            }
        }
    }
    return ans
}

private fun buildBlockWhere(p: IntArray, blocks: List<Block>): IntArray {
    val n = p.size
    val where = IntArray(n) { -1 }
    for ((blockID, b) in blocks.withIndex()) {
        for (index in b.from..<b.to) {
            where[p[index]] = blockID
        }
    }
    return where
}

private fun List<Int>.subOneArray() = map { if (it >= 0) it - 1 else it }.toIntArray()

private fun solve() {
    readInt()
    val p1 = readInts().subOneArray()
    val p2 = readInts().subOneArray()
    readInt()
    val d1 = readInts().subOneArray()
    val d2 = readInts().subOneArray()
    when (val ans = solve(p1, p2, d1, d2)) {
        null -> println("Inconsistent")
        else -> println(ans.joinToString("") { it.toString() })
    }
}

fun main() {
    repeat(readInt()) { solve() }
}

private fun buildBlocks(inv: IntArray, d: IntArray): List<Block>? {
    val n = inv.size
    val elements = buildList {
        add(IndexedValue(-1, -1))
        addAll(d.asList().mapIndexedNotNull { index, value -> if (value >= 0) IndexedValue(index, inv[value]) else null })
        add(IndexedValue(d.size, n))
    }
    val blocks = buildList {
        for (i in 1..<elements.size) {
            val left = elements[i - 1]
            val right = elements[i]
            add(Block(left.value + 1, right.value, right.index - left.index - 1))
            if (left.value > right.value) return null
            if (right.value in 0..<n) {
                add(Block(right.value, right.value + 1, 1))
            }
        }
    }
    return blocks
}
